/* -*- C++ -*-
 *
 * ReplicaLocationIndex.cpp
 *
 * Copyright (c) 2002 CERN on behalf of the EU DataGrid.
 * For license conditions see LICENSE file or 
 * http://www.edg.org/license.html
 * 
 */


#ifndef ReplicaLocationIndex_H
#define ReplicaLocationIndex_H 1

#include <string>
#include <set>
#include <map>
#include <vector>

#include <EdgReplicaLocationIndex/ReplicationExceptions.h>

namespace EdgReplicaLocationIndex
{
  /** 
	 * A Replica Location Index, which allows you to final
	 * <code>EdgLocalReplicaCatalog</code>s which contain mappings for the given
	 * <code>guid</code>s.
	 * @author <a href="mailto:Radovan.Chytracek@cern.ch">Radovan Chytracek</a>
	 * @author <a href="mailto:james.casey@cern.ch">James Casey</a>
	 * @author <a href="mailto:levi.lucio@cern.ch">Levi Lucio</a>
	 * @version $Id: ReplicaLocationIndex.h,v 1.4 2003/10/31 14:04:14 pkunszt Exp $
	 */
		
	typedef std::vector<std::string> LRCs;
	typedef unsigned char            Byte;
	typedef std::vector<Byte>        Bytes;
	
	class ReplicaLocationIndex
	{
			/**
			 * This is an instance of a Metadata Catalog, which stores mappings
			 * between GUIDs and logical file names (LFNs or Aliases).
			 */
    
		public:
			/** create a new ReplicaLocationIndex.
			 */
			ReplicaLocationIndex(const std::string& url) { }
    
			/** Standard virtual destructor
			 */
			virtual ~ReplicaLocationIndex() { }

			/** get the URL this ReplicaLocationIndex resides at
			 *
			 * @return the URL
			 */
			virtual const std::string getUrl() const =0;

			/**
			 * Given an guid, return the list of LRCs  which
			 * contains a mapping for the guid.
			 * @param guid the guid
			 * @param toFill the vector of LRCs to be filled if any found
			 * @throws CommunicationException in case of an unknown server side error
			 */
			virtual void getLRC(const std::string& guid, LRCs& toFill ) const
				throw(CommunicationException) = 0;

			/** Return the list of LRCs which publish to this RLI
			 *
			 * @param toFill the list of LRCs to be filled if any
			 * @throws CommunicationException in case of an unknown server side error
			 */
			virtual void lrcList(LRCs& toFill) const
				throw(CommunicationException) = 0;
			
			/** Does this RLI know about a given guid.
			 *
			 * @param guid the guid
			 * @return <code>true</code> if it does.
			 * @throws CommunicationException in case of an unknown server side error
			 */
			virtual bool exists(const std::string& guid) const
				throw(CommunicationException) = 0;
			
			/** Does this RLI have information about a given LRC containing a
			 * mapping for a given guid.
			 * @param lrc the lrc
			 * @param guid the guid
			 * @return <code>true</code> if it does.
			 * @throws CommunicationException in case of an unknown server side error
			 */
			virtual bool existsAt(const std::string& lrc,
														const std::string& guid) const
				throw(CommunicationException) = 0;
			
			/*
			 * LRC - RLI Communication API
			 */
			
			/** update the information we have for an LRC with a new set of
			 * information.
			 * @param lrcEndpoint The URL of the endpoint of the lrc.
			 * @param expiration the time in millis when this filter should expire
			 * @param bits
			 * @param entries
			 * @param numBits
			 * @param optimalNumKeys
			 * @param numHashes
			 * @throws CommunicationException in case of an unknown server side error
			 */
			virtual void update(const std::string& lrcEndpoint,
													long expiration,
													const Bytes& bits,
													int entries,
													int numBits,
													int optimalNumKeys,
													int numHashes)
				throw(CommunicationException) = 0;
			
			/** remove the filter associated with a given endpoint
			 *
			 * @param lrcEndpoint the endpoint to remove
			 * @return <code>true</code> if the endpoint existed in the RLI
			 * @throws CommunicationException in case of an unknown server side error
			 */
			virtual bool remove(const std::string& lrcEndpoint)
				throw(CommunicationException) = 0;
			
			/*
			 * admin API
			 */
			

      /** Check if a server is alive. It returns a stats string.
			 * If everything is ok, this will start with <code>OK</code> else
			 * it will start with <code>FAIL</code>.
			 * @return the stats string
			 * @throws CommunicationException in case of an unknown server side error
			 */
			virtual const std::string ping() const
				throw(CommunicationException) = 0;
			
			/** Return RLI server version as std::string
			 * @throws CommunicationException in case of an unknown server side error
			 */
			virtual const std::string getVersion() const
				throw(CommunicationException) = 0;

			/** Return RLI server interface version as std::string
			 * @throws CommunicationException in case of an unknown server side error
			 */
			virtual const std::string getInterfaceVersion() const
				throw(CommunicationException) = 0;
	};
}

#endif // ReplicaLocationIndex_H

			 
