/*
 *
 * ReplicaLocationIndexImpl.h
 *
 * Copyright (c) 2002 CERN on behalf of the EU DataGrid.
 * For license conditions see LICENSE file or 
 * http://www.edg.org/license.html
 * 
 */


#ifndef ReplicaLocationIndexImpl_H
#define ReplicaLocationIndexImpl_H 1

#include <string>
#include <vector>
#include <memory>
#include <iostream>

#include "EdgReplicaLocationIndex/ReplicaLocationIndex.h"
#include "EdgReplicaLocationIndex/ReplicationExceptions.h"
#include "edg_gsoap_baseH.h"

namespace EdgReplicaLocationIndex
{
  /** 
   *
   * @author <a href="mailto:james.casey@cern.ch">James Casey</a>
   * @author <a href="mailto:Radovan.Chytracek@cern.ch">Radovan Chytracek</a>
   * @version $Id: ReplicaLocationIndexImpl.h,v 1.6 2004/05/05 08:55:22 jamesc Exp $
   */
 
  class ReplicaLocationIndexImpl : public ReplicaLocationIndex {
    
    public:
      /* create a new RemoteLocalReplicaCatalog
       * @param url  The URL of the LocalReplicaCatalog to connect to.
       * This will normally be a normal http/s URI which points
       * to a SOAP endpoint.
       */
      ReplicaLocationIndexImpl(const std::string& url);
        
#ifdef WITH_WP2_SECURITY
      ReplicaLocationIndexImpl(const std::string& url,
			       char *passphrase,
			       char *credentials,
			       char *cacertdir,
			       char *rseedfile);

      ReplicaLocationIndexImpl(const std::string& url,
			       char *passphrase,
			       char *credentials,
			       char *cacertdir,
			       char *rseedfile,
			       char *role,
			       char *policy,
			       char *defaultRole,
			       char *defaultPolicy);
#endif
        
      virtual ~ReplicaLocationIndexImpl();

      const std::string getUrl() const;

			/**
			 * Given an guid, return the list of LRCs  which
			 * contains a mapping for the guid.
			 * @param guid the guid
			 * @param toFill the vector of LRCs to be filled if any found
			 * @throws CommunicationException in case of an unknown server side error
			 */
			virtual void getLRC(const std::string& guid, LRCs& toFill ) const
				throw(CommunicationException);

			/** Return the list of LRCs which publish to this RLI
			 *
			 * @param toFill the list of LRCs to be filled if any
			 * @throws CommunicationException in case of an unknown server side error
			 */
			virtual void lrcList(LRCs& toFill) const
				throw(CommunicationException);
			
			/** Does this RLI know about a given guid.
			 *
			 * @param guid the guid
			 * @return <code>true</code> if it does.
			 * @throws CommunicationException in case of an unknown server side error
			 */
			virtual bool exists(const std::string& guid) const
				throw(CommunicationException);
			
			/** Does this RLI have information about a given LRC containing a
			 * mapping for a given guid.
			 * @param lrc the lrc
			 * @param guid the guid
			 * @return <code>true</code> if it does.
			 * @throws CommunicationException in case of an unknown server side error
			 */
			virtual bool existsAt(const std::string& lrc,
														const std::string& guid) const
				throw(CommunicationException);
			
			/*
			 * LRC - RLI Communication API
			 */
			
			/** update the information we have for an LRC with a new set of
			 * information.
			 * @param lrcEndpoint The URL of the endpoint of the lrc.
			 * @param expiration the time in millis when this filter should expire
			 * @param bits
			 * @param entries
			 * @param numBits
			 * @param optimalNumKeys
			 * @param numHashes
			 * @throws CommunicationException in case of an unknown server side error
			 */
			virtual void update(const std::string& lrcEndpoint,
													long expiration,
													const Bytes& bits,
													int entries,
													int numBits,
													int optimalNumKeys,
													int numHashes)
				throw(CommunicationException);
			
			/** remove the filter associated with a given endpoint
			 *
			 * @param lrcEndpoint the endpoint to remove
			 * @return <code>true</code> if the endpoint existed in the RLI
			 * @throws CommunicationException in case of an unknown server side error
			 */
			virtual bool remove(const std::string& lrcEndpoint)
				throw(CommunicationException);
			
			/*
			 * admin API
			 */
			
      /** Check if a server is alive. It returns a stats string.
       * If everything is ok, this will start with <code>OK</code> else
       * it will start with <code>FAIL</code>.
       * @return the stats string
			 * @throws CommunicationException in case of an unknown server side error
       */
      virtual const std::string ping() const
				throw(CommunicationException);
			
      /** Return RLI server version as std::string
			 * @throws CommunicationException in case of an unknown server side error
       */
      virtual const std::string getVersion() const
				throw(CommunicationException);
			
      /** Return RLI server interface version as std::string
			 * @throws CommunicationException in case of an unknown server side error
       */
      virtual const std::string getInterfaceVersion() const
				throw(CommunicationException);
			
    private:

      //
      // helper methods
      //

      void
        dump_soap(int ret, const struct soap* soap) const;

      void
        dump_soap_error(int ret, const struct soap* const_soap) const;

      void
        handleGenericFaults(int ret, const struct soap* soap) const;

      const char*
        endpoint() const;

      struct soap*
        soap() const;

      //
      // member variables
      //
      const std::string m_endpoint;
      struct soap*      m_soap;
  };
}


#endif // ReplicaLocationIndexImpl_H

// EOF ReplicaLocationIndexImpl.h

